{# This will display the version of the docs as a badge

Colors from:

Wong, B. Points of view: Color blindness.
Nat Methods 8, 441 (2011). https://doi.org/10.1038/nmeth.1618

#}

<a href="https://docs.scipy.org/doc/">
    {% if "dev" in version %}
        {# orange for dev #E69F00 #}
        <img src="https://img.shields.io/badge/release-{{ version|replace("+", "%2B") }}-E69F00?style=for-the-badge"
         style="margin-left: 1em; width: 60%;">
    {% elif versionwarning %}
        {# red for old #980F0F #}
        <img src="https://img.shields.io/badge/release-{{ version|replace("+", "%2B") }}-980F0F?style=for-the-badge"
         style="margin-left: 1em; width: 60%;">
    {% else %}
        {# green for stable #009E73 #}
        <img src="https://img.shields.io/badge/release-{{ version }}-009E73?style=for-the-badge"
         style="margin-left: 1em; width: 60%;">
    {% endif %}
</a>