/*=============================================================================
    Copyright (c) 1998-2008 Joel de Guzman
    Copyright (c) 2001-2008 Hartmut Kaiser
    Copyright (c) 2001-2003 Daniel Nuffer
    Copyright (c) 2002-2003 Martin Wille
    Copyright (c) 2002 Juan Carlos Arevalo-Baeza
    Copyright (c) 2002 Raghavendra Satish
    Copyright (c) 2002 Jeff Westfahl
    Copyright (c) 2001 Bruce Florman
    Copyright (c) 2003 Giovanni Bajo
    Copyright (c) 2003 Vaclav Vesely
    Copyright (c) 2003 Jonathan de Halleux
    http://spirit.sourceforge.net/
    http://www.boost.org/libs/spirit

  Distributed under the Boost Software License, Version 1.0. (See accompanying
  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

  See http://www.boost.org/libs/spirit for documentation
=============================================================================*/
#if !defined(BOOST_SPIRIT_CLASSIC_APRIL_11_2008_0849AM)
#define BOOST_SPIRIT_CLASSIC_APRIL_11_2008_0849AM

#include <boost/spirit/home/classic/core.hpp>
#include <boost/spirit/home/classic/meta.hpp>
#include <boost/spirit/home/classic/error_handling.hpp>
#include <boost/spirit/home/classic/iterator.hpp>
#include <boost/spirit/home/classic/symbols.hpp>
#include <boost/spirit/home/classic/utility.hpp>
#include <boost/spirit/home/classic/attribute.hpp>

#endif
