// Boost.Bimap
//
// Copyright (c) 2006-2007 Matias Capeletto
//
// Distributed under the Boost Software License, Version 1.0.
// (See accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt)

//  See www.boost.org/libs/bimap for documentation.

// Convenience header

#include <boost/bimap/bimap.hpp>

namespace boost
{
    using ::boost::bimaps::bimap;
}

