// Copyright (C) 2004-2009 The Trustees of Indiana University.

// Use, modification and distribution is subject to the Boost Software
// License, Version 1.0. (See accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt)

//  Authors: Nick Edmonds
//           Andrew Lumsdaine

#ifndef BOOST_GRAPH_USE_MPI_HPP
#define BOOST_GRAPH_USE_MPI_HPP

#define BOOST_GRAPH_USE_MPI

#endif // BOOST_GRAPH_USE_MPI_HPP
