@import "bootstrap/variables.less";

.spc-rightsidebar {
	color: @gray;
	.navigation {
		padding: @paddingSmall;
		font-size: @fontSizeSmall;
	}
	.navigation .nav-title {
		font-weight: bold;
		text-transform: uppercase;
	}
	.navigation li { margin: 5px; }
}	